##
## __init__.py
## Login : <glibersat@linux62.org>
## Started on  Sat Dec 31 00:54:08 2005 Guillaume Libersat
## $Id$
## 
## Copyright (C) 2005 Guillaume Libersat
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##

from reader import GPBinReader
from header import GP4PieceInfo, GP4Lyrics
from midi import GP4Midi, GP4MidiChannel
from body import GP4Measure, GP4Marker

class GP4Piece(object):
    def dump(self):
        self.info.dump()
        self.lyrics.dump()
        self.midi.dump()
        
""" Read a Guitar Pro 4 File """
class GP4Reader:
    """ Open a GP4 file from VFS """
    def fromFile(self, path):
        fd = open(path, "rb")

        piece = GP4Piece()
        reader = GPBinReader(fd)

        self._read_headers(reader, piece)
        self._read_body(reader, piece)
        
        fd.close()

        return piece

    """ Read Headers """
    def _read_headers(self, reader, piece):
        """ Read file version """
        def read_version():
            length = reader.read_byte()
            
            piece.file_version = reader.read_string(length)
            reader.seek(0x1F)

        def read_piece_info():
            piece.info = GP4PieceInfo()
            
            piece.info.title = reader.read_long_string()
            piece.info.subtitle = reader.read_long_string()
            piece.info.interpret = reader.read_long_string()
            piece.info.album = reader.read_long_string()
            piece.info.author = reader.read_long_string()
            piece.info.copyright = reader.read_long_string()
            piece.info.tabber = reader.read_long_string()
            piece.info.instructions = reader.read_long_string()

            nb_notice = reader.read_long()
            piece.info.notice = ""
            for i in range(0, nb_notice):
                piece.info.notice += reader.read_long_string()

            piece.info.tripletfeel = reader.read_bool()

        def read_lyrics():
            piece.lyrics = GP4Lyrics()
            piece.lyrics.track_id = reader.read_long()

            for i in range(0, 5):
                piece.lyrics.setLine(i, reader.read_lyrics_string())


        def read_miscinfo():
            piece.info.tempo = reader.read_long()
            piece.info.key = reader.read_byte()
            piece.info.octave = reader.read_long()

        def read_midi():
            piece.midi = GP4Midi()

            for port in range(1, 5):
                for channel in range(1, 17):
                    ch = GP4MidiChannel()
                    
                    ch.instrument = reader.read_long()
                    ch.volume = reader.read_byte()
                    ch.balance = reader.read_byte()
                    ch.chorus = reader.read_byte()
                    ch.reverb = reader.read_byte()
                    ch.phaser = reader.read_byte()
                    ch.tremolo = reader.read_byte()

                    # GP3 compat
                    reader.read_byte()
                    reader.read_byte()

                    piece.midi.setChannel(port * channel, ch)

        def read_tracks_measures():
            piece.info.measures = reader.read_long()
            piece.info.tracks = reader.read_long()

        # Header
        read_version()
        read_piece_info()
        read_lyrics()
        read_miscinfo()
        read_midi()
        read_tracks_measures()
        

    def _read_body(self, reader, piece):
        def read_measures():
            for i in range(piece.info.measures):
                print "Measure num", i
                meas = GP4Measure()
                
                d = reader.read_byte()

                # Numerator
                if d & 0x1:
                    meas.numerator = reader.read_byte()

                # Denominator
                if d & 0x2:
                    meas.denominator = reader.read_byte()

                # Begin of repeat
                if d & 0x4:
                    meas.begin_repeat = True

                # End of repeat
                if d & 0x8:
                    meas.repeat_count = reader.read_byte()

                # Alternate endings
                if d & 0x10:
                    meas.altending_count = reader.read_byte()

                # Marker
                if d & 0x20:
                    meas.marker = GP4Marker()
                    print reader.fd.tell()
                    meas.marker.name = reader.read_long_string()
                    print meas.marker.name
                    meas.marker.color = reader.read_byte()

                # Tonality
                if d & 0x40:
                    meas.tonality = reader.read_byte()

                # Double bar
                if d & 0x80:
                    meas.doublebar = True

        read_measures()
